
/*******************************************************************************
NAME:     adapsp.h 
PURPOSE:  Prototypes of ADAPS support functions.  
	  Must be #include'd after las.h and adaps.h 
          The bulk of this file was created by running makep.csh and makep.C
	  makep.C is under $CVSROOT/bin/prototypes/


FILE HISTORY
--------------------------------------------------------------------------------
  Date       Programmer                          Changes 
---------  ---------------  ----------------------------------------------------
 Jun 1996  David Zokaites   original development
 Oct 1996  David Zokaites   updated 3 prototypes, see srf 897 
*******************************************************************************/








extern "C"
{

// prototypes of ADAPS supports by category

// acqio
   int c_acqclose (const long *fdesc);
   int c_acqhead (const long *fdesc, const long *year, const char *station, 
		 const long *dtype, struct SATPASS *pass);
   int c_acqinfo (const char *station, const long *img_typ, long *skip_head,
                 long *rec_line, long *byte_line, long *mnr_ptr, long *mnr_siz,
                 long *tip_ptr, long *tip_siz, long *img_ptr, long *img_siz,
                 long *img_fmt, long *swap_flg, long *flip_flg, char *sta_name);
   int c_acqopen (long *fdesc, const char *file, const long *access,
                 const char *station, const long *img_typ, long *tip_flg);
   int c_acqread (const long *fdesc, short *mnrbuf, short *tipbuf,
                 short *imgbuf);
   int c_acqskip (const long *fdesc, const long nrecs);
   int c_acqtmcor (const long *satnum, const long *f_month, const long *f_day,
                  const long *f_year, const long *f_hour, const long *f_min,
                  const long *f_sec, const long *f_fsec, long *timcor);
   void c_bil2bip (const short *ibuf, short *obuf, const long *nsamps,
                  const long bands[], const long *fill);
   void c_bip2bil (const short *ibuf, short *obuf, const long *nsamps,
                  const long bands[], const long *fill);
   int c_tipopen (int *fd, const char *file, const long *access);
   int sharp_mnr (const unsigned char *inbuf, short *mnrbuf);

// adaps
   void c_cvtdms (const double *latlon, long *deg, long *mins, long *sec);
   int c_getdirs (const char *filespec, char dir_names[][CMLEN], long *ndirs,
                 const long maxdirs, const long first_call);
   int c_getfiles (const char *filespec, char file_names[][CMLEN], long *nfiles,
                  const long maxfiles, const long first_call,
                  const long ck_lock);
   void c_getscene (const struct MODEL *model, char *sceneid);
   void c_getsizes (const char file_names[][CMLEN], const long nfiles,
                   double file_sizes[]);
   void c_wdbclose (const FILE *fp_seq, const FILE *fp_rel);
   int c_wdbopen (const char *filenam, FILE **fp_seq, FILE **fp_rel);
   int c_wdbrank (const long ftype, const long fcount, const long feature[],
                 long ranks[]);
   int c_wdbrel (const FILE *fp_rel, const long *total_pts, long *num_pts,
                float pnts[]);
   int c_wdbseq (const FILE *fp_seq, const long ranks[], const long crossflg,
                const float *minlat, const float *minlon, const float *maxlat,
                const float *maxlon, long *total_pts, const FILE *fp_rel);
   void c_wvsfree (const struct WVS_VECTOR *wvs_vector);
   int c_wvsquery (const char *path, const struct WVS_COVERAGE *wvs_coverage,
                  struct WVS_VECTOR *wvs_vector, const double lllat,
                  const double lllon, const double urlat, const double urlon);
   int ccupdt (const long curlin, const short mnr_frm[], const long illum,
              long ocoeffs[], long coeffs[], long lastcoeffs[], long prtcnt[],
              long cntsum[], long prtsmp[], long prtndx[], long *nxtprt,
              const long prtpop, const long *sampls, long intsum[],
              long spcsum[], const long start, const long cpriod,
              const double aij[][5], const double bi[], const double nsp[],
              const double bnmrp[][60], const double enmr[][60], const long n[],
              const struct MODEL *model);
   void findpolecross (long *maxline, long *minline, const struct MODEL *model);
   int findrange (const double lat1, const double lat2, long startln[2],
                 long stopln[2], const long quadnum, const long areaopt,
                 long *numfound, const struct MODEL *model);
   int kaufcals (const char *hostbl, const long satnum, const long ayear,
                const long aday, float Kval[], long ocoeffs[]);
   int osscals (const char *hostbl, const long satnum, const long ayear,
               const long aday, float Kval[], long ocoeffs[]);
   int precals (const char *hostbl, const long satnum, const long ayear,
               const long aday, float Kval[], long ocoeffs[]);
   void sunearthdis (const long jday, float *aus);

// archio
#ifdef ARCHIO_H // these prototypes won't compile unless archio.h is included
   void c_todrop (const struct DROPPED *drop, char buffer[][DRP_SIZ+1],
                 long *nbuf);
   void c_tometa (const struct MODEL *model,
                 char meta_data[META_LINES][HDR_SIZ+1]);
#endif
   int c_arclose (const char *image, const long *arcdesc); 
   int c_arcopnrd (const char *image, const long *access, long window[4],
                  const struct MODEL *model, const long *tpunit, long *arcdesc);
   int c_arcopnwrt (const char *image, const long *dtype, const long *nlines, 
		   long *arcdesc); 
   int c_arcread (const long *arcdesc, short **mnr, short **samples);
   int c_arcseek (const long *arcdesc, const long line);
   int c_arcwrite (const long *arcdesc, const short mnr[],
                  const short samples[]);
   int c_getaddr (const char *host_img, struct MODEL *model);
   int c_getahdr (const char *arch_name, struct MODEL *model,
                 struct DROPPED *drop);
   int c_hdrclose (const int *fd);
   int c_hdropen (int *fd, const char *file, const long *access);
   int c_hdrread (const char buffer[][HDR_SIZ+1], struct MODEL *model,
                 struct DROPPED *drop);
   int c_hdrwrite (const int *out_fd, const struct MODEL *model,
                  const struct DROPPED *drop);
   int c_putaddr (const char *host_img, const struct MODEL *model);
   int c_putahdr (const char *arch_name, const struct MODEL *model,
                 const struct DROPPED *drop);
   void c_setatt (const char buffer[][HDR_SIZ+1], struct MODEL *model);
   int c_setdrop (const char buffer[][HDR_SIZ+1], struct DROPPED *drop);
   void c_sethead (const char buffer[][HDR_SIZ+1], struct MODEL *model,
                  const long version);
   void c_toatt (const struct MODEL *model, char buffer[][HDR_SIZ+1]);
   void c_toephem (const struct MODEL *model, char buffer[]);
   void c_tohead (const struct MODEL *model, char buffer[][HDR_SIZ+1]);
   int c_tpgetahdr (const char *sceneid, const char *comment, long *tpunit,
                   struct MODEL *model, struct DROPPED *drop);

// geolib
   int c_copygrid (const struct GRID *ingrid, struct GRID *outgrid);
   int c_freegrid (struct GRID *grid);
   int c_geo2ls (const double *ullat, const double *ullon, const double *pdistx,
                const double *pdisty, const double *pixlat,
                const double *pixlon, long *pixline, long *pixsamp);
   int c_prtgrid (const char *hostname, const struct GRID *grid);
   int c_readgrid (const char *hostname, struct GRID *grid);
   int c_wrtgrid (const struct GRID *grid);

// model
   void brouwcorrect (const double dpele[], double oselem[]);
   void brouwer (const double *t, const struct MODEL *model, double *truea,
                double *rsat, double *thetas, double *anlng, double *inclin,
                long *zdot, long *eorbit, long *porbit, double *slat,
                double *slong);
   void c_coords (struct MODEL *model);
   void c_daynite (const long *jdate, const long *year, const double *tsec,
                  const double *plat, const double *plong, double *sunzen,
                  long *dayn);
   void c_forward (const double *line, const double *sample,
                  const double *tarelev, const struct MODEL *model,
                  double *tarlatd, double *tarlong, double view_angles[]);
   void c_getorb (const double *tsec, const struct MODEL *model, long *orbit,
                 long *passdir, double *eqxing);
   int c_initmod (const long *satnum, const long *year, const long *month,
                 const long *day, const long *shour, const long *smin,
                 const long *ssec, const long *sfrac, const long *dtype,
                 const long *nlines, struct MODEL *model);
   void c_inverse (const double *tarlatd, const double *tarlong,
                  const double *tarelev, const struct MODEL *model,
                  double *line, double *sample, double view_angles[]);
   void c_ll2ls (const double *lat, const double *lng, const double *elev,
                const struct MODEL *model, float *line, float *sample,
                double view_angles[]);
   void c_ls2ll (const float *line, const float *sample, const double *elev,
                const struct MODEL *model, double *lat, double *lng,
                double view_angles[]);
   int c_readephem (const long *satnum, const long *year, const long *month,
                   const long *day, char *eline);
   void c_setephem (const char *eline, struct MODEL *model);
   void c_setsat (struct MODEL *model);
   void c_sunpos (const long *jdate, const long *year, const double *tsec,
                 double *sunlat, double *sunlng);

// products
   int c_pg2struct (const char *str, struct ORDER *order);
   int c_pgpoints (const struct MODEL *model, const long npixels, long *q);
   int c_pgread (struct DESCRIPT *descript, long *orders_q);
   int c_pgtest (const struct MODEL *model, const struct ORDER *order,
                const long points_q, long *start, long *nlines);
   int c_pgwrite (const struct DESCRIPT *descript, const long orders_q);

// queue
   long c_qcreate (const char *order, const char *duplicates);
   long c_qdelete (const long q, const long position);
   long c_qdestroy (const long q);
   long c_qempty (const long q);
   long c_qfind (const long q, const void *objin, void **objout,
                const long (compare)(), long *position);
   long c_qfirst (const long q, void **ptr);
   long c_qinsert (const long q, const void *object, const long (compare)());
   long c_qnext (const long q, void **ptr);
   long c_qposition (const long q, const long pos, void **ptr);
   long c_qprev (const long q, void **ptr);
   long c_qprint (const long q, const void (print_obj)());
   long c_qsize (const long q);

// util
   void c_cvtime (long *tsec, const long *hour, const long *min,
                 const long *sec);
   void c_cvtmil (const long *milsec, long *hour, long *min, long *sec,
                 long *fsec);
   void c_cvtsec (const long *tsec, long *hour, long *min, long *sec,
                 const long *fsec);
   int c_getsat (const char *key, long *satnum, char *satnam, char *satno,
                char *satid, char *satcode);
   void c_grejul (const long *year, const long *month, const long *day,
                 long *jdate);
   int c_isleap (const long *year);
   void c_julgre (const long *year, const long *jdate, long *month, long *day);
   int c_lockfil (const char *hname, const long *lockflg);
   void c_pck32 (const short *inbuf, unsigned char *outbuf, const long *nwords);
   void c_pixflip (short pixels[], const long nvalues);
   int c_pixscale (const long *idtype, const long *odtype,
                  const unsigned char *in, unsigned char *out, const long *ns,
                  const float iscale, const float ioffset, const float oscale,
                  const float ooffset, const long *rndopt);
   void c_unpck32 (const unsigned char *inbuf, short *outbuf,
                  const long *nwords, const long *swap);
   int c_wrtlog (const char *logfile, const char *key, const char *code,
                const char *text);


} // end extern "C"



