#ifndef ADAPSSUP_H
#define ADAPSSUP_H

#include "imageio.h"
#include "calcoef.h"
#include "archio.h"
#include "avhrr.h"
#include "model.h"
#include "wvs.h"

/* Prototype functions to be called by any ADAPS support routine or module   */

int c_appcoeff
(
    struct MODEL *model,	/* AVHRR model information		*/
    struct CALSTR *calstr,	/* Calibration structure		*/
    long window[],	/* Image window specification			*/
    long channel[],	/* Channels to process				*/
    long maxchan,	/* Maximum number of channels to process	*/
    char *minmax,	/* Min/max flag					*/
    double omin[NCHANS],	/* min pixel value array		*/
    double omax[NCHANS],	/* max pixel value array		*/
    unsigned char *buffer,    /* image I/O buffer                       */
    long offset1[MAXIMG*NCHANS],/* Offsets into "buffer" for input bands*/
    long offset2[NCHANS],    /* Offsets into "buffer" for output bands	*/
    struct COEFFSTR *coeff	/* Calibration coefficients		*/
);

int c_calcoeff
(
    long line,			/* INPUT: Line being processed		     */
    long mline,			/* INPUT: Image master line number	     */
    short min_frm[MNR_WORDS],	/* INPUT: Buffer containing minor frame data */
    int minfd,			/* INPUT: Minor frame file descriptor	     */
    struct MODEL *model,	/* INPUT: Model and orbital information	     */
    struct COEFFSTR *coeff,	/* INPUT/OUTPUT: All coeff (optical, thermal)*/
    struct CALSTR *calstr	/* OUTPUT: Calibration parameters structure  */
);

void c_cvtdms
(
    double *latlon,	/* Latitude or longitude of value to convert */
    long *deg,          /* Degrees */
    long *mins,		/* Minutes */
    long *sec		/* Seconds */
);

int c_getdirs
(
    char *filespec,           /* File specification for location of subdirs */
    char dir_names[][CMLEN],  /* Array of directory names to be returned    */
    long *ndirs,              /* Number of directories found                */
    long maxdirs,             /* Max number of directories to return        */
    long first_call           /* Flag for first call to this function (0,1) */
);

int c_getkvals
(
    long satnum,	/* IN  - Satnum (0=tiros-n, 1=noaa-1, 2=noaa-2, ...)  */
    char ocalmeth[],	/* IN  - Type kaufman, prelaunch, teillet, vermote,rao*/
    double kvals[]	/* OUT - K values returned			      */
);

int c_getscale
(
    long option,	/* IN  - Scaling option (US_BYTE, US_I2, US_I4, ...)  */
    char value[],	/* IN  - String indicating values being scaled        */
    double *minimum,	/* OUT - Lowest valid value after shifting data       */
    double *maximum,	/* OUT - Largest valid value after shifting data      */
    double *scale,	/* OUT - Scaling factor for the specified option      */
    double *unscale,	/* OUT - Unscaling factor for the specified option    */
    double *offset	/* OUT - Offset value for the specified option        */
);

void c_getscaleopt
(
    char *scale_opt,	/* Scaling option (US,GLOBAL,NONE)	*/
    long odtype,	/* Output data type			*/
    long *soption	/* Scaling option (US_I2,AVHRR_BYTE,etc)*/
);

void c_getscene
(
    struct MODEL *model,	/* Model and orbital information	*/
    char *sceneid		/* Scene id				*/
);

void c_getsizes
(
    char file_names[][CMLEN],	/* Full path specification of files   */
    long nfiles,		/* Number of files in file_names      */
    double file_sizes[]       	/* Size in bytes of each file         */
);

void c_wdbclose
(
    FILE *fp_seq,		/* file pointer to the WDBII sequential file */
    FILE *fp_rel 		/* file pointer to the WDBII relative file   */
);

int c_wdbopen
(
    char *filenam,		/* file root name to be opened		     */
    FILE **fp_seq,		/* file pointer to the WDBII sequential file */
    FILE **fp_rel 		/* file pointer to the WDBII relative file   */
);

int c_wdbrank
(
    long ftype,		/* type of feature 			*/
    long fcount,		/* number of features			*/
    long feature[],	/* feature classifications to process	*/
    long ranks[]	/* sorted rankings desired from WDBII   */
);

int c_wdbrel
(
    FILE *fp_rel,	/* file pointer to the WDBII relative file	     */
    long *total_pts,	/* total number of points in the line segment	     */
    long *num_pts,	/* number of points to process at this time          */
    float pnts[]	/* latitude/Longitude point defining the segments    */
);

int c_wdbseq
(
    FILE *fp_seq,	/* file pointer to the WDBII sequential file	     */
    long ranks[],	/* ranking of interest				     */
    long crossflg,	/* indicates crossing Internat'l Date Line (180 long)*/
    float *minlat,	/* minimum latitude of area of interest		     */
    float *minlon,	/* minimum longitude of area of interest	     */
    float *maxlat,	/* maximum latitude of area of interest		     */
    float *maxlon,	/* maximum longitude of area of interest	     */
    long *total_pts,	/* total number of points in the line segment	     */
    FILE *fp_rel 	/* file pointer to the WDBII relative file	     */
);

int c_wvsquery
(
    char *path,				/* location of WVS files	*/
    char *wvsfile,			/* WVS file name		*/
    struct WVS_VECTOR *wvs_vector,	/* resulting vector information	*/
    double lllat,			/* bounding rect - minlat	*/
    double lllon,			/*		   minlon	*/
    double urlat,			/*		   maxlat	*/
    double urlon 			/*		   maxlon	*/
);

int c_wvsfile
(
    double coord[],
    long *area,
    long *subarea
);

int c_wvsfree
(
    struct WVS_VECTOR *wvs_vector  /* WVS coverage structure to be freed */
);

int findrange
(
    double lat1,                /* first lat in range                   */
    double lat2,                /* second lat in range                  */
    long startln[2],            /* line corresponding to first lat      */
    long stopln[2],             /* line corresponding to second lat     */
    long quadnum,               /* quad number                          */
    long areaopt,               /* areas to look for                    */
    long *numfound,             /* number of areas found                */
    struct MODEL *model         /* Model and orbital information        */
);


#endif
