
#ifndef LEVEL1BIO_H
#define LEVEL1BIO_H

#include "model.h"
#include "avhrr.h"

/* Define Level-1b Format I/O constants
-----------------------------------------*/

#define UNP_LAC_1  2272     /* Record size (bytes): 1 band unpacked LAC data  */
#define UNP_LAC_2  4320     /* Record size (bytes): 2 band unpacked LAC data  */
#define UNP_LAC_3  6368     /* Record size (bytes): 3 band unpacked LAC data  */
#define UNP_LAC_4  8416     /* Record size (bytes): 4 band unpacked LAC data  */
#define UNP_LAC_5 10464     /* Record size (bytes): 5 band unpacked LAC data  */
#define UNP_GAC_1  2536     /* Record size (bytes): 1 band unpacked GAC data  */
#define UNP_GAC_2  4168     /* Record size (bytes): 2 band unpacked GAC data  */
#define UNP_GAC_3  5808     /* Record size (bytes): 3 band unpacked GAC data  */
#define UNP_GAC_4  7440     /* Record size (bytes): 4 band unpacked GAC data  */
#define UNP_GAC_5  9080     /* Record size (bytes): 5 band unpacked GAC data  */
#define PCK_LAC_5  7400     /* Record size (bytes): 5 band packed LAC data    */
#define PCK_GAC_5  6440     /* Record size (bytes): 5 band packed GAC data    */

/* NOTE: all record sizes confirmed */
#define	KLM_UNP_LAC_1	6144	/* rec size(bytes): 1 band unpacked LAC data  */
#define	KLM_UNP_LAC_2	10240	/* rec size(bytes): 2 band unpacked LAC data  */
#define	KLM_UNP_LAC_3	14336	/* rec size(bytes): 3 band unpacked LAC data  */
#define	KLM_UNP_LAC_4	18432	/* rec size(bytes): 4 band unpacked LAC data  */
#define	KLM_UNP_LAC_5	22528	/* rec size(bytes): 5 band unpacked LAC data  */
#define KLM_PCK_LAC_5	15872	/* rec size(bytes): 5 band packed LAC data    */
#define	KLM_UNP_GAC_1	2360	/* rec size(bytes): 1 band unpacked GAC data  */
#define	KLM_UNP_GAC_2	3176	/* rec size(bytes): 2 band unpacked GAC data  */
#define	KLM_UNP_GAC_3	3992	/* rec size(bytes): 3 band unpacked GAC data  */
#define	KLM_UNP_GAC_4	4816	/* rec size(bytes): 4 band unpacked GAC data  */
#define	KLM_UNP_GAC_5	5632	/* rec size(bytes): 5 band unpacked GAC data  */
#define KLM_PCK_GAC_5	4608	/* rec size(bytes): 5 band packed GAC data    */

/* NOTE: sizes V_UNP_GAC_1, V_UNP_GAC_3 and V_UNP_GAC_5 include the
zero fill to even word boundary.
-------------------------------------------------------------------*/
#define V_UNP_LAC_1  4096    /* Video size (bytes): 1 band unpacked LAC data  */
#define V_UNP_LAC_2  8192    /* Video size (bytes): 2 band unpacked LAC data  */
#define V_UNP_LAC_3 12288    /* Video size (bytes): 3 band unpacked LAC data  */
#define V_UNP_LAC_4 16384    /* Video size (bytes): 4 band unpacked LAC data  */
#define V_UNP_LAC_5 20480    /* Video size (bytes): 5 band unpacked LAC data  */
#define V_UNP_GAC_1   820    /* Video size (bytes): 1 band unpacked GAC data  */
#define V_UNP_GAC_2  1636    /* Video size (bytes): 2 band unpacked GAC data  */
#define V_UNP_GAC_3  2456    /* Video size (bytes): 3 band unpacked GAC data  */
#define V_UNP_GAC_4  3272    /* Video size (bytes): 4 band unpacked GAC data  */
#define V_UNP_GAC_5  4092    /* Video size (bytes): 5 band unpacked GAC data  */
#define V_PCK_LAC_5 13656    /* Video size (bytes): 5 band packed LAC data    */
#define V_PCK_GAC_5  2728    /* Video size (bytes): 5 band packed GAC data    */

#define PREVIDEO    448      /* Bytes in line preceding video data            */
#define KLMPREVIDEO 1264     /* Bytes in line preceding video data            */

#define MAXRECSIZ KLM_UNP_LAC_5	/* Largest record size                        */
#define MAXLINSIZ KLM_UNP_LAC_5	/* Largest line size			      */	

#define MOSTBND       5     /* Max bands for level 1B tape                    */
#define TBM_HDR_SIZ 122     /* Record size (bytes): TBM header record         */
#define ARS_HDR_SIZ 512     /* Record size (bytes): ARS header record         */
#define TELOFF      308     /* Offset to telemetry data format                */
#define KLM_TELOFF  1056    /* Offset to telemetry data format                */

#define TBM_PRESENT      1     /* Flag signaling tbm header present           */
#define ARS_PRESENT      2     /* Flag signaling ars header present           */
#define TBM_NOTPRES     -1     /* Flag signaling tbm header not present       */

#ifndef DEG
#define DEG(x)    (x) * DEG_PER_RAD
#endif


/*     Table of values defining Level-1B format
       -------------------------------------------

type:	Data type defining LAC/GAC, packed/unpacked, and number of channels
	present.

pre-video:	Number of bytes of information preceding the video data.

video:	Total number of bytes of video data, including the trailing bytes that
	must be zero-filled to end at an even word boundary.

0-fill:	Number of trailing bytes of video data that must be zero-filled prior
	to loading the actual video data.  Any of these bits not eventually
	containing actual data will then be zero-filled.

post-video:	Number of bytes of information following the video data.

spare:	Number of spare bytes to fill out line.


type   	          pre-video    video      (0-fill)  post-video spare
--------------------------------------------------------------------
UNP_LAC_1         PREVIDEO   V_UNP_LAC_1     (0)        0        0
UNP_LAC_2         PREVIDEO   V_UNP_LAC_2     (0)        0        0
UNP_LAC_3         PREVIDEO   V_UNP_LAC_3     (0)        0        0
UNP_LAC_4         PREVIDEO   V_UNP_LAC_4     (0)        0        0
UNP_LAC_5         PREVIDEO   V_UNP_LAC_5     (0)        0        0
UNP_GAC_1         PREVIDEO   V_UNP_GAC_1     (2)        0        0
UNP_GAC_2         PREVIDEO   V_UNP_GAC_2     (0)        0        0
UNP_GAC_3         PREVIDEO   V_UNP_GAC_3     (2)        0        0
UNP_GAC_4         PREVIDEO   V_UNP_GAC_4     (0)        0        0
UNP_GAC_5         PREVIDEO   V_UNP_GAC_5     (2)        0        0
PCK_LAC_5         PREVIDEO   V_PCK_LAC_5     (4)       22      674
PCK_GAC_5         PREVIDEO   V_PCK_GAC_5     (4)       22       22

KLM table
type   	          pre-video    video      (0-fill)  post-video spare
--------------------------------------------------------------------
KLM_UNP_LAC_1  KLMPREVIDEO   V_UNP_LAC_1     (8)       560     216
KLM_UNP_LAC_2  KLMPREVIDEO   V_UNP_LAC_2     (8)       560     216
KLM_UNP_LAC_3  KLMPREVIDEO   V_UNP_LAC_3     (8)       560     216
KLM_UNP_LAC_4  KLMPREVIDEO   V_UNP_LAC_4     (8)       560     216
KLM_UNP_LAC_5  KLMPREVIDEO   V_UNP_LAC_5     (8)       560     216
KLM_UNP_GAC_1  KLMPREVIDEO   V_UNP_GAC_1     (4)       152     120
KLM_UNP_GAC_2  KLMPREVIDEO   V_UNP_GAC_2     (4)       152     120
KLM_UNP_GAC_3  KLMPREVIDEO   V_UNP_GAC_3     (2)       152     118
KLM_UNP_GAC_4  KLMPREVIDEO   V_UNP_GAC_4     (8)       152     120
KLM_UNP_GAC_5  KLMPREVIDEO   V_UNP_GAC_5     (4)       152     120
KLM_PCK_LAC_5  KLMPREVIDEO   V_PCK_LAC_5     (8)       568     376
KLM_PCK_GAC_5  KLMPREVIDEO   V_PCK_GAC_5     (8)       152     456

End of table */

/* Prototype functions to be called by any ADAPS support routine or module    */

void c_l1b2l1b
(
    long irecsiz,		/* Input buffer record size	*/
    long ichan[],		/* Input channels within inbuf	*/
    unsigned char *ibuf,	/* Input buffer			*/
    long orecsiz,		/* Output buffer record size	*/
    long ochan[],		/* Output channels within obuf	*/
    unsigned char *obuf		/* Output buffer		*/
);

int c_l1bclosd
(
FILE		*fd 		/* IN: file descriptor of file to close	*/
);

int c_l1bclost
(
long		*tun,		/* IN: tape logical unit number		*/
long		*tape 		/* OUT: tape open flag			*/
);

int c_l1bmkhdr
(
    struct MODEL *model,	/* IN: AVHRR model structure		*/
    struct DROPPED *drop,	/* IN: dropped line structure		*/
    long window[],		/* IN: window for model SL,SS,EL,and ES	*/
    long chans[],		/* IN: selected channels		*/
    long *recsiz,		/* IN: record size in bytes		*/
    unsigned char *buf,		/* OUT: pointer to TBM/ARS buffer	*/
    unsigned char *dsh		/* OUT: pointer to DSH buffer		*/
);

int c_l1bopndr
(
char infile[CMLEN],     /* INPUT:  Input file				*/
FILE	**in_fd,	/* OUTPUT: File descriptor to the input file	*/
long	*recsiz,	/* OUTPUT: Record size of data read		*/
unsigned char buf[],	/* OUTPUT: TBM/ARS header buffer		*/
unsigned char dsh[],	/* OUTPUT: DSH header buffer			*/
struct MODEL *model 	/* OUTPUT: AVHRR model information		*/
);

int c_l1bopndw
(
char		outfile[CMLEN],	/* IN: output file name			*/
long		*recsiz,	/* IN: data record size in bytes	*/
unsigned char	*buf,		/* IN: TBM/ARS header buffer		*/
unsigned char	*dsh,		/* IN: DSH header buffer		*/
long		*hsize,		/* IN: TBM/ARS header size		*/
FILE		**out_fd 	/* OUT: output file descriptor		*/
);

int c_l1bopntr
(
    char drive[CMLEN],	 /* INPUT : Tape drive name			*/
    long *tun,		 /* OUTPUT: Tape unit number			*/
    unsigned char buf[], /* OUTPUT: TBM/ARS header buffer		*/
    unsigned char dsh[], /* OUTPUT: DSH header buffer			*/
    long *recsiz,	 /* OUTPUT: Record size of data read           	*/
    struct MODEL *model, /* OUTPUT: AVHRR model information		*/
    long *tape		 /* OUTPUT: Flag for tape open			*/
);

int c_l1bopntw
(
    char drive[CMLEN],	/* IN: drive name			*/
    char *density,	/* IN: tape density			*/
    long *tun,		/* OUT: tape logical unit number	*/
    unsigned char *buf,	/* IN: pointer to TBM/ARS header buffer	*/
    unsigned char *dsh,	/* IN: pointer to DSH buffer		*/
    long *recsiz,	/* IN: record size, DSH size, in bytes	*/
    long *tape		/* IN: tape open flag			*/
);

int c_l1breadd
(
FILE		*in_fd,		/* IN: file pointer to input file	*/
long		*recsiz,	/* IN: Record size of data to read	*/
unsigned char	*recptr 	/* OUT: Pointer to Level 1b scan line	*/
);

int c_l1breadt
(
    long *tun,			/* IN: Tape logical unit number		*/
    long *recsiz,		/* IN: Record size in bytes		*/
    char comment[][CMLEN],	/* IN: Tape description			*/
    long *volnum,		/* IN/OUT: Volume number		*/
    long *tape,			/* IN/OUT: Tape open flag		*/
    unsigned char *recptr	/* OUT: Pointer to Level 1b scan record	*/
);

void c_l1brecsiz
(
    long dtype,			/* Data type GAC/LAC/HRPT	(input)	*/
    long pack_flg,		/* Pack flag TRUE/FALSE		(input)	*/
    long klm_flg,		/* NOAA KLM series flag (T/F)   (input)	*/
    long nbands,		/* Number of channels 1-5	(input)	*/
    long *recsiz		/* Level-1B record size (bytes)	(output)*/
);

int c_l1bseekd
(
FILE		*in_fd,			/* IN: file descriptor		*/
long		window[],		/* IN: line number to seek	*/
long		*recsiz,		/* IN: record size in bytes	*/
struct MODEL	*model,			/* IN: AVHRR model structure	*/
long		*drop_line 		/* OUT: number of drop lines	*/
);

int c_l1bseekt
(
    long *tun,			/* IN: tape unit number		*/
    long window[],		/* IN: line number to seek	*/
    long *recsiz,		/* IN: record size in bytes	*/
    struct MODEL *model,	/* IN: AVHRR model structure	*/
    char comment[][CMLEN],	/* IN: tape description(s)	*/
    long *volnum,		/* IN/OUT: tape volume number	*/
    long *tape,			/* IN/OUT: tape open flag	*/
    long *drop_line		/* OUT: actual scan line number	*/
);

int c_l1bskipt
(
    char drive[CMLEN],	/* INPUT: Tape drive name		  */
    long *skip		/* INPUT: Number of files to skip ( >= 0) */
);

int c_l1bsumm
(
char	rootname[CMLEN],/* Root name of the summary file	*/
char	acctnum[10],	/* account number			*/
char	ordernum[14],	/* order number				*/
char	unitnum[5],	/* unit number				*/
char	prodesc[18],	/* product description			*/
char	density[5],	/* tape density				*/
struct	MODEL	*model,	/* Model and orbital information	*/
char	lasout[CMLEN] 	/* Full LAS output name for summary	*/
);

int c_l1buphdr
(
FILE		*out_fd,	/* IN: output file descriptor		*/
struct MODEL	*model,		/* IN: model structure			*/
struct DROPPED	*drop,		/* IN: dropped line structure		*/
long		window[],	/* IN: window for model			*/
long		chans[],	/* IN: channels for the output		*/
long		*recsiz 	/* IN: record size for output in bytes	*/
);

int c_l1bwrited
(
FILE		*out_fd,	/* IN: output file descriptor		*/
unsigned char	*recptr,	/* IN: pointer to Level 1b data line	*/
long		*recsiz 	/* IN: record size in bytes for output	*/
);

int c_l1bwritet
(
    long *tun,			/* IN: tape logical unit number		*/
    unsigned char *recptr,	/* IN: pointer to level 1b scan record	*/
    long *recsiz,		/* IN: record size in bytes		*/
    long *tape			/* IN/OUT: tape open flag		*/
);


#endif

