/******************************************************************************
FUNCTION:	FINDALIAS

PURPOSE:	Searches the user's alias file for the specified alias name.
		If the alias name is found, the associated text is returned.

PROGRAM HISTORY:
Version    Date     Author           Change Request
-------    -----    -------------    --------------
  1.0      10/86    C. Greenhagen     New Development
  1.1      12/87    B. Ailts          Change include dir. spec.
  5.0      04/88    D. Hollaren       changed include spec to las
				      return E_SUCC/E_FAIL
  5.1      11/90    D. Etrheim        Corrected for alias of length 9.  Need
				      ASZ+2 to handle lf and NULL chars.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE 
	Does NOT print any error messages prior to returning with E_FAIL.

PROJECT			LAS

ALGORITHM 
Read each record from the user's ALIAS LIST file.
    When a match is found return the alias text.

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <math.h>

#ifdef vms
#include <file.h>
#else
#include <sys/types.h>
#include <sys/file.h>
#endif

#include "las.h"
#include "alias.h"


/**
 * findalias:
 * @aliasf:  I: alias file
 * @aliasname: I: alias name
 * @aliastext: O: alias text
 *
 * Searches the user's alias file for the specified alias name.
 * If the alias name is found, the associated text is returned.
 *
 * Returns: E_SUCC:  Successful completion.
 *          E_FAIL:  Unsuccessful completion.
 */
int findalias 
(
    FILE *aliasf,		/* alias file  (input)		*/
    char *aliasname,	/* alias name  (input)		*/
	char *aliastext 	/* alias text  (output)		*/
)
{
#define ASZ	9			/* max length of alias name	*/
#define TSZ	CMLEN-1			/* max length of alias text	*/

char
    buf[ASZ+2],				/* buffer for reads		*/
    *fptr;				/* pointer to alias file	*/
int
    len,				/* string length 		*/
    foundmatch;				/* found alias name match t/f	*/
    
foundmatch = 0;

while (foundmatch == 0)				/* read alias file,	 */
    {						/* search for alias name */
    fptr = fgets (buf,ASZ+2,aliasf);
    if (fptr == NULL)
	return (E_FAIL);

    len = strlen (buf);
    buf[len-1] = 0;

    if (strcmp (aliasname,buf) == 0)		/* found a match,	*/
	{					/* read alias text	*/
	foundmatch = 1;
	fptr = fgets (aliastext,TSZ+1,aliasf);
	if (fptr == NULL)
	    return (E_FAIL);
	else
	    {
	    len = strlen (aliastext);
	    aliastext[len-1] = 0;
	    }
	}
    else					/* not a match,		*/
	{					/* skip alias text	*/
	fptr = fgets (aliastext,TSZ+1,aliasf);
	if (fptr == NULL)
	    return (E_FAIL);
	}
    }

return (E_SUCC);
}
