/******************************************************************************
NAME:	SQUEEZE

PURPOSE:
	SQUEEZE returns a pointer to a null terminated character string.  It
	searches for the first non-blank character of the input parameter
	"str" from the right.  If the first non-blank character is not
	the null value (0), then the next character is assigned to the null
	value.  This routine is used to null terminate a fortran character
	string.

	Type definitions are defined as follows:
		unsigned char	==  unsigned 8-bit integer
		short	==  signed 16-bit integer
		int	==  signed 32-bit integer
		float	==  signed 32-bit real
		char	==  character

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         5/86       K. Gacke     initial development
    1.1        10/86       K. Gacke     if a " " is inputted, the returned
				        string is null
    1.2	       12/87	   B. Ailts	change to raw 'C' types
					change include directory specifications
    1.2	       12/92	   T. Mittan    Corrected problem in for loop.  If
					a string of blanks are used ptr would
					be trying to look before the allocated
					string.  Corrected error flag	

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:
	Search from the right of a character string for the first non blank
	character.  If this character is not the null value, then it is 
	assigned to the null value.  This routine is used to null terminate
	fortran character strings.

******************************************************************************/

#include <stdio.h>
#include <string.h>

#include "tae.h"
#include "worgen.h"

/**
 * squeeze:
 * @str: I: String to be "squeezed"
 * @len: I: Length of str.
 *
 * SQUEEZE returns a pointer to a null terminated character string.  It
 * searches for the first non-blank character of the input parameter
 * "str" from the right.  If the first non-blank character is not
 * the null value (0), then the next character is assigned to the null
 * value.  This routine is used to null terminate a fortran character
 * string.
 * 
 * Type definitions are defined as follows:
 * 	unsigned char	==  unsigned 8-bit integer
 * 	short	==  signed 16-bit integer
 * 	int	==  signed 32-bit integer
 * 	float	==  signed 32-bit real
 * 	char	==  character
 *
 * Returns: "Squeezed" string
 */
char *squeeze
(
    const char *str,
    int len
)
{
long non_fatal = 1;
char *newstr;
char *ptr;

newstr = malloc(len+1);
if (newstr == NULL)
   c_errmsg("Error allocating dynamic memory","err-alloc",&non_fatal);

strncpy(newstr,str,len);
for(ptr = newstr + len - 1; ((ptr >= newstr) && (*ptr == ' ')); ptr--)
	;
*(ptr + 1) = '\0';
return(newstr);
}
