#ifndef LABSERV_H
#define LABSERV_H

int c_labclose
(
    int fd,               /* I: File descriptor of opened file */
    const char *hostname, /* I: host file name */
    int action            /* I: action flag: 0=close, 1=crunch, 2=delete */
);

int c_labdelete
(
    int fd,             /* I: file descriptor */
    const char *inkey   /* I: key of record to be deleted */
);

void c_lsmknm
(
    const char *inname, /* I: input host image name */
    const char *ext,    /* I: input extension for label services file */
    char *outname       /* O: output host label services file name */
);

/* returns the opened file descriptor, or E_FAIL */
int c_labopen
(
    const char *hname,  /* I: TAE host file name */
    int rwmode          /* I: File access: READ, WRITE, APPEND, or RW */
);

int c_labread
(
    int fd,             /* I: file descriptor */
    char *key,          /* I/O: key of record to be read, or key read if
                                any key requested */
    int *clen,          /* I/O: number of char bytes to be read */
    int *dlen,          /* I/O: number of data bytes to be read */
    char *cbuf,         /* O: character buffer to be read */
    void *dbuf,         /* O: data buffer to be read */
    char *dtype         /* O: datatype of data record (B,I2,I4,R4,R8) */
);

int c_labreplace
(
    int fd,             /* I: file descriptor */
    const char *key,    /* I: key to be written */
    int clen,           /* I: number of char bytes to be written */
    int dlen,           /* I: number of data bytes to be written */
    const char *cbuf,   /* I: char buffer to be written */
    const void *dbuf,   /* I: data buffer to be written */
    const char *dtype   /* I: data type of data record (B,I2,I4,R4,R8) */
);

int c_labrew
(
    int fd              /* I: file descriptor */
);

int c_labstat
(
    const char *hostname,   /* I: host file name */
    int mode                /* I: type of access mode to check */
                            /*  F_OK = Tests to see if the file exists */
                            /*  X_OK = Execute */
                            /*  W_OK = Write (implies delete access) */
                            /*  R_OK = Read */
);

int c_labwrite
(
    int fd,             /* I: file descriptor */
    const char *key,    /* I: key of record to be written */
    int clen,           /* I: number of char bytes to be written */
    int dlen,           /* I: number of data bytes to be written */
    const char *cbuf,   /* I: character buffer to be written */
    const void *dbuf,   /* I: data buffer to be written */
    const char *dtype   /* I: datatype of data record (B,I2,I4,R4,R8) */
);

#ifndef NO_OBSOLETE_INTERFACE

int c_lsclos
(
    const long *fd,         /* I: File descriptor of opened file */
    const char *hostname,   /* I: host file name */
    const long *action      /* I: action flag: 0=close, 1=crunch, 2=delete */
);

int c_lsdel
(
    const long *fd,     /* I: file descriptor */
    const char *inkey   /* I: key of record to be deleted */
);

int c_lsopen
(
    long *fd,           /* O: File descriptor of open file */
    const char *hname,  /* I: TAE host file name */
    const long *rwmode  /* I: File access: READ, WRITE, APPEND, or RW */
);

int c_lsread
(
    const long *fd,     /* I: file descriptor */
    char *key,          /* I/O: key of record to be read, or key read if
                                any key requested */
    long *clen,         /* I/O: number of char bytes to be written */
    long *dlen,         /* I/O: number of data bytes to be written */
    char *cbuf,         /* O: character buffer to be written */
    void *dbuf,         /* O: data buffer to be written */
    char *dtype         /* O: datatype of data record (B,I2,I4,R4,R8) */
);

int c_lsrepl
(
    const long *fd,         /* I: file descriptor */
    const char *key,        /* I: key to be written */
    const long *clen,       /* I: number of char bytes to be written */
    const long *dlen,       /* I: number of data bytes to be written */
    const char *cbuf,       /* I: char buffer to be written */
    const void *dbuf,       /* I: data buffer to be written */
    const char *dtype       /* I: data type of data record (B,I2,I4,R4,R8) */
);

int c_lsrew
(
    const long *fd          /* I: file descriptor */
);

int c_lsstat
(
    const char *hostname,   /* I: host file name */
    const long *mode        /* I: type of access mode to check */
                            /*   0 = Tests to see if the file exists */
                            /*   1 = Execute */
                            /*   2 = Write (implies delete access) */
                            /*   4 = Read */
);

int c_lswrit
(
    const long *fd,     /* I: file descriptor */
    const char *key,    /* I: key of record to be written */
    const long *clen,   /* I: number of char bytes to be written */
    const long *dlen,   /* I: number of data bytes to be written */
    const char *cbuf,   /* I: character buffer to be written */
    const void *dbuf,   /* I: data buffer to be written */
    const char *dtype   /* I: datatype of data record (B,I2,I4,R4,R8) */
);

#endif /* obsolete interfaces */

#endif
