#ifndef _LAMS_H_
#define _LAMS_H_

#include "radpak.h"

/*  These are the constants for the LAMS software -- all begin with the 
    letter 'M' to signify 'mosaic'
-----------------------------------------------------------------------------*/
#define MMXCOL 35    /* Maximum number of columns in a tabular file          */
#ifdef gould
#define MMXROW 32767 /* Maximum number of rows in a tabular file 	     */
#else
#define MMXROW 99999 /* Maximum number of rows in a tabular file 	     */
#endif

#define MMXPPR  7500 /* Maximum number of pairs of points in a cutline	     */
#define MMXPT	15000 /* Maximum number of points in a cutline(MMXPPR * 2)    */
#define MEMXPT  15000 /* Maximum extended number of points in a graphics file */

#define MMXIMG    50 /* Maximum mosaic input images			     */

#define MOPIX	   4 /* Error code signifying that too many pixels lie
			outside of the image				     */
struct  MFDESC
   {
   long fd;
   long reclen;		/* record length -- number of entries in record	     */
   long numrec;		/* number of records				     */
   long acc;		/* access type					     */
   };

/* Prototypes */

int c_getrgd
(
    char *host_name,            /* Host file name of the mapping grid file */
    struct RADGRID *grid        /* Geometric mapping grid                  */
);

int c_grclos
(
    struct MFDESC **fdesc,
    char *hostin
);

int c_gropen
(
    struct MFDESC **fdesc,
    char *hostin,
    long *reclen,
    long *nrec,
    long *acc
);

int c_grread
(
    struct MFDESC **fdesc,
    long *rec,
    float *buffer
);   

int c_grwrit
(
    struct MFDESC **fdesc,
    long *rec,
    float *buffer
);   

int c_putrgd
(
    char *host_name,            /* Host file name of the mapping grid file */
    struct RADGRID *grid        /* radiometric mapping grid                */
);

int c_rdsca
(
    char *hname,
    long *sca
);

int c_rgdred
(
    struct RADGRID *grid,       /* Geometric mapping grid             */
    double *tolerance           /* Tolerance for linear approximation */
);

int c_tbclos
(
    struct MFDESC **fdesc,
    char *hostin
);

int c_tbopen
(
    struct MFDESC **fdesc,
    char *hostin,
    long *nrow,
    long *ncol,
    long *acc
);  

int c_tbrcol
(
    struct MFDESC **fdesc,
    long *col,
    float *buffer
);   

int c_tbrrow
(   
    struct MFDESC **fdesc,
    long *col,
    long *ncol,
    long *row,
    float *buffer
);

int c_tbwcol
(
    struct MFDESC **fdesc,
    long *col,
    float *buffer
);   

int c_tbwrow
(
    struct MFDESC **fdesc,
    long *col,
    long *ncol,
    long *row,
    float *buffer
);

#endif  /* _LAMS_H_ */
