#ifndef LASFILEIO_H
#define LASFILEIO_H

/* This header file defines the prototypes for file reading and writing 
   routines that take care of reading and writing files in the LAS standard
   byte order (endian) regardless of the endian order of the local machine. */

#include <unistd.h>  /* size_t definition */
#include <stdio.h>   /* FILE definition */

size_t c_fread
(
   void *buf,               /* I/0: pointer to buffer to read the data into */
   size_t size,             /* I: size of each element read */
   size_t count,            /* I: number of elements to read */
   FILE *stream             /* I: stream to read */
);

size_t c_fwrite
(
   const void *buf,         /* I: pointer to the buffer of data to write */
   size_t size,             /* I: size of each element read */
   size_t count,            /* I: number of elements to read */
   FILE *stream             /* I/O: stream to read */
);

ssize_t c_read
(
   int fd,                  /* I: file descriptor to read */
   void *buf,               /* I/0: pointer to buffer to read the data into */
   size_t count,            /* I: total number of bytes to read */
   size_t size              /* I: size of each element read so they can be
                                  byte swapped if needed */
);

ssize_t c_write
(
   int fd,                  /* I/0: file descriptor to read */
   const void *buf,         /* I: pointer to the buffer of data to write */
   size_t count,            /* I: total number of bytes to write */
   size_t size              /* I: size of each element read so they can be
                                  byte swapped if needed */
);

#endif
