/*
 *	Valid message types.
 *	Note: ENQ and QUIT are not real message types; they are
 *	used for communication with the pdsqmgr.
 */
enum message_type {INVALID, ORD, VALD, COMP, REJD, ENQ, QUIT, INPR, CLEN, QC, ERR};

/*
 *	The pcs_info structure contains all the fields in the
 *	message passed from the PCS after the file has been
 *	decoded.  All character strings are null terminated.
 */
struct pcs_info
	{
	int	format_version;		/* version of message file */
	enum message_type message_type;	/* type of message */
	char	order_nbr[14];		/* order number */
	char	unit_nbr[5];		/* unit number (zero filled numeric) */
	char	message_date[11];	/* date the message was sent */
	char	*host_name;		/* host machine data is on */
	char	product_code[5];	/* unique code identifying product */
	int	priority_code;		/* priority: 1 => high, 9 => low */
	char	date_due[11];		/* date the unit is due */
	char	project_id[16];		/* project ID */
	char	input_media_type[3];	/* type of input media */
	char	input_media_format[3];	/* digital archive media process fmt */
	char	**storage_loc;		/* storage location(s), null term */
	char	ordering_id[19];	/* unique order ID */
	char	*spec_instr_text[4];	/* special instructions */
	char	spectral_bands[9];	/* bands */
	char	product_format[5];	/* output format: NDF, CEOS, etc */
	char	product_media[3];	/* output media: 9T, 8M, etc */
	char	product_density[5];	/* output density: 6250, high, low... */
	char	ansi_label_f;		/* Y/N ANSI labelled */
	char	data_organization[4];	/* BIL, BSQ */
	char	tape_blocking[6];	/* tape blocking factor */
	char	byte_ordering[4];	/* MSB or LSB */
	char	*input_dir;		/* directory name of input file */
	char	create_col_fire_f;	/* Y/N create a ColorFire product */
	char	retain_dem_f;		/* Y/N copy DEM to customer prod. */
	char	compress_fill_f;	/* Y/N compress fill data */
	int	number_copies;		/* number of customer copies */
	char	bin_nbr[4];		/* bin in dissemination */
	char	*return_addr;		/* e-mail address to reply to */
	char	*e_mail_address;		/* e-mail to notify customer */
	struct pcs_info *next;		/* next pcs_info structure */
	struct pcs_info *prev;		/* previous pcs_info structure */
	};

struct PDS_STRUCT
	{
	struct pcs_info pcs_info;
	char	output_media[10];
	char	status[10];
	char	tapedrive[20];
	char	ordernumber[14];
	char	number_units;
	};

struct pcs_info *pcs_info_alloc();
struct pcs_info *pcs_info_dup(struct pcs_info *);
int pcs_info_free(struct pcs_info *);
struct pcs_info *pcsin(FILE *);
struct pcs_info *pdsget(int);
void pcs_out(FILE *, struct pcs_info *);
int pdsconn(char *, int);
