#ifndef UTIL_H
#define UTIL_H

#include <stdio.h>
#include <signal.h>
#include "imageio.h"             /* MAXBND definition                       */
#include "cm.h"                  /* PRS_LST definition                      */
#include "ddr.h"                 /* DDR definition                          */
#include "util2.h"
#include "worgen.h"

void las_abort
(
    struct PARBLK *block,        /* I: TAE parameter block                  */
    const char message[],        /* I: Error message                        */
    const char key[],            /* I: Error message key                    */
    const char prog_name[],      /* I: Program name                         */
    const char files[][CMLEN],   /* I: Files to be deleted                  */
    int num_files                /* I: Number of files to be deleted        */
);

int c_block_cleanup
(
    sigset_t *oldmask            /* O: Status of signal mask before 
                                       SIGTERM was masked to be blocked     */
);

int c_bndchk
(
    const char *host,            /* I: Input host filename                  */
    int  *inbnds,                /* I/O: Number of bands specified          */
    int  *vlbnds,                /* O: Valid window values for bands        */ 
    int  *nbands                 /* O: Number of bands per image            */
);

void las_clean
(
    struct PARBLK *vblock,       /* I: TAE parameter block                  */
    const char *pname,           /* I: Program name                         */
    const char hnames[][CMLEN],  /* I: Output file names                    */
    int count                    /* I: Number of output files               */
);

void las_clean_noexit
(
    struct PARBLK *vblock,       /* I: TAE parameter block                  */
    const char hnames[][CMLEN],  /* I: Output file names                    */
    int count                    /* I: Number of output files               */
);

int c_calc_bound_ls_coords
(
    double *ulx,                 /* I/O: upper left x                       */
    double *lrx,                 /* I/O: lower right x                      */
    double *uly,                 /* I/O: upper left y                       */
    double *lry,                 /* I/O: lower right y                      */
    int  units,                  /* I: window units                         */
    int  proj_code,              /* I: GCTP projection code                 */
    int  zone_code,              /* I: zone code                            */
    const double proj_coef[15],  /* I: GCTP projection parameters           */
    int  out_units,              /* I: output units                         */
    int  datum_code,             /* I: GCTP datum code                      */
    double pdist_y,              /* I: projection distance/pixel(y-direction)*/
    double pdist_x,              /* I: projection distance/pixel(x-direction)*/
    double *pxmin,               /* O: minimum x value                      */
    double *pxmax,               /* O: maximum x value                      */
    double *pymin,               /* O: minimum y value                      */
    double *pymax                /* O: maximum y value                      */
);

int c_check_window
(
    const struct DDR *ddr,       /* I: Pointer to DDR structure             */
    const double inwind[],       /* I: Input window to be checked           */
    int  vlwind[]                /* O: Valid window values                  */
);

void c_creat_name
(
    char *temp_file,             /* I/O: Temp file for name created         */
    const char *ext              /* I: Default extension                    */ 
);


void c_delfil
(
    const char *host             /* I: Host file name to be deleted         */
);

int c_get_multiple_outs
(
    struct PARBLK *vblock,       /* I: TAE parameter block                  */
    int max_num_of_outs,         /* I: Maximum number of output files that
                                       can be retrieved                     */
    const char *param,           /* I: TAE parameter name                   */
    char hosout[][CMLEN],        /* O: Host image file name(s)              */
    int *number_of_outs          /* O: Total count of output files          */
);

int c_getcnt
(
    struct PARBLK *block,        /* I: TAE parameter block                  */
    const char *param_buf,       /* I: Input image parameter name           */
    int  *count                  /* O: Element array number of param        */
);


int las_getin
(
    struct PARBLK *block,        /* I: TAE parameter block                  */
    const char *param_buf,       /* I: Input image parameter name           */
    int  element,                /* I: Element array number of param        */
    char tnames[][CMLEN],        /* O: TAE names of images specified        */
    char hnames[][CMLEN],        /* O: Host names of images specified       */
    int  *images,                /* O: Number of images                     */
    int (*window)[4],            /* O: Image windows (sl ss nl ns)          */
    int (*bands)[MAXBND+1],      /* O: Image bands                          */
    int *nbands,                 /* O: Number of bands per image            */
    int *totbnd,                 /* O: Total number of bands specified      */
    int  imglimit,               /* I: Maximum images caller can handle     */
    int  *same_window,           /* O: 1 if all windows the same            */
    int  (*qflag)[2]             /* O: 1 user defined window & bands else
                                       0 if default window and bands        */
);


int las_getin_nocheck
(
    struct PARBLK *block,        /* I: TAE parameter block                  */
    const char *param_buf,       /* I: Input image parameter name           */
    int         element,         /* I: Element array number of param.       */
    char tnames[][CMLEN],        /* O: TAE names of images specified        */
    char hnames[][CMLEN],        /* O: Host names of images specified       */
    int  *images,                /* O: Number of images                     */
    int  (*window)[4],           /* O: Image windows (sl ss nl ns)          */
    int  (*bands)[MAXBND+1],     /* O: Image bands                          */
    int  *nbands,                /* O: Number of bands per image            */
    int  *totbnd,                /* O: Total number of bands specified      */
    int  imglimit,               /* I: Maximum images caller can handle     */
    int  *same_window,           /* O: 1 if all windows the same            */
    int  (*qflag)[2]             /* O: 1 user defined window & bands else
                                       0 if default window and bands        */
);

int las_getin2
(
    const char inparm[],         /* I:  Input image string with             */
    char tnames[][CMLEN],        /* O:  TAE names of images specified       */
    char hnames[][CMLEN],        /* O:  Host names of images specified      */
    int  *images,                /* O:  Number of images                    */
    int  (*window)[4],           /* O:  Image windows (sl ss nl ns)         */
    int  (*bands)[MAXBND+1],     /* O:  Image bands                         */
    int  *nbands,                /* O:  Number of bands per image           */
    int  *totbnd,                /* O:  Total number of bands specified     */
    int  imglimit,               /* I:  Maximum images caller can handle    */
    int  *same_window,           /* O:  1 if all windows the same           */
    int  (*qflag)[2]             /* O:  1 user defined window & bands else
                                        0 if default window and bands       */
);

int las_getnam
(
    struct PARBLK *vblock,       /* I: TAE parameter block                  */
    const char *param,           /* I: TAE parameter name                   */
    int element,                 /* I: element of parameter                 */
    char *fname,                 /* O: Host file name to be returned        */
    const char *ext,             /* I: default extension                    */
    int access                   /* I: Type of I/O access                   */
);

int las_getout
(
    struct PARBLK *vblock,       /* I: TAE parameter block                  */
    const char *param,           /* I: TAE parameter name                   */
    char hostout[][CMLEN]        /* O: Host image file name(s)              */
);

int las_gettyp
(
    const char hosin[][CMLEN],   /* I: Input host file names                */
    int nmrimg,                  /* I: Number of input images               */
    int  *odtype                 /* O: Output data type of input img        */
);


int las_history_close
(
    int fd,                      /* I: File descriptor of in images hist    */
    const char hosin[]           /* I: Input host name                      */
);

int las_history_open
(
    const char hosin[],          /* I: Input image name                     */
    int access                   /* I: Type of access to input image hist   */
);

int las_history_read
(
    int  fd,                     /* I: File descriptor of in image hist     */
    char key[],                  /* I/O: Key buffer                         */
    char **buf,                  /* O: History buffer                       */
    int  *len                    /* I: Char length of label service rec     */
);

int las_hstout
(
    struct PARBLK *vblock,       /* I: Pointer to TAE parameter block       */
    const char *hosout,          /* I: Image history to be updated          */
    const char (*hosin)[CMLEN],  /* I: Image names whose hist is copied     */
    int   numin,                 /* I: # of images specified in "hosin"     */
    const char *histext          /* I: Text to be included in hist record   */
);

int c_imgprs
(
    char *i_specs,               /* I: Image specifications                 */
    struct PRS_LST *list,        /* I: List of input image names            */
    int *images                  /* I/O: Total number of input images       */
);

int las_inmnmx
(
    int dtype,                   /* I: Data type                            */
    int totbnd,                  /* I: Total number of bands                */
    double *minval,              /* O: Minimum intensity values array       */
    double *maxval               /* O: Maximum intensity values array       */
);

int c_install_cleanup_handler
(
    void (*sig_hand)(int)        /* I: Signal handler function              */
);

int las_mkimg
(
    const char *in_string,       /* I: Character string containing filename */
    const char *ext,             /* I: Defines string to be appended        */
    char *hname,                 /* O: String containing hostname of image  */
    int flag                     /* I: Flag indicating existence of file    */
);

int las_mknam
(
    const char *filename,        /* I: Input file name                      */
    char *outfile,               /* O: Host file name to be returned        */
    const char *ext,             /* I: Default extension                    */
    int access                   /* I: Type of I/O access                   */
);

int c_remove_cleanup_handler (void);

int c_setprt
(
    struct PARBLK *vblock,       /* I: TAE parameter block                  */
    int  *term,                  /* O: Boolean for terminal output          */
    int  *file,                  /* O: Boolean for file output              */
    int  *lp,                    /* O: Boolean for line printer output      */
    char *prtfile,               /* I/O: File name or lp working file       */
    char *filename               /* O: File name or lp output file          */
);

void c_splitname
(
    const char filespec[CMLEN],  /* I: File specification                   */
    char directory[CMLEN],       /* O: Directory portion of filespec        */
    char root[CMLEN],            /* O: Root portion of the file name        */
    char extension[CMLEN]        /* O: Extension portion of the file        */
);

int c_unblock_cleanup
(
    const sigset_t *oldmask      /* I: Status of signal mask before SIGTERM
                                       was masked to be blocked             */
);

int c_wndchk
(
    const char *host,            /* I: Host name                            */
    const double inwind[],       /* I: Input window to be checked           */
    int vlwind[]                 /* O: Valid window                         */
);

int c_wndconv
(
    const char hname[],          /* I: Host name of image specified         */
    double window[],             /* I/O: Image window 
                                         On input: uly, ulx, lry, lrx
                                         On output: sl, ss, nl, ns          */
    const int units[]            /* I: Window units                         */
);

int c_window_proj_to_ls
(
    const struct DDR *ddr,       /* I: Pointer to DDR structure             */
    double window[],             /* I/O: Image window
                                         On input: uly, ulx, lry, lrx
                                         On output: sl, ss, nl, ns          */
    const int units[]            /* I: Window units                         */
);



#ifndef NO_OBSOLETE_INTERFACE

void c_abort
(
    struct PARBLK *block,        /* I: TAE parameter block                  */
    const char message[],        /* I: Error message                        */
    const char key[],            /* I: Error message key                    */
    const char prog_name[],      /* I: Program name                         */
    const char files[][CMLEN],   /* I: Files to be deleted                  */
    const long *num_files        /* I: Number of files to be deleted        */
);

void c_clean
(
   struct PARBLK *vblock,        /* I: TAE parameter block                  */
   const char *pname,            /* I: Program name                         */
   const char hnames[][CMLEN],   /* I: Output file names                    */
   const long *count             /* I: Number of output files               */
);

int c_getin
(
    struct PARBLK *block,        /* I: TAE parameter block                  */
    const char *param_buf,       /* I: Input image parameter name           */
    const long *element,         /* I: Element array number of param        */
    char tnames[][CMLEN],        /* O: TAE names of images specified        */
    char hnames[][CMLEN],        /* O: Host names of images specified       */
    long *images,                /* O: Number of images                     */
    long (*window)[4],           /* O: Image windows (sl ss nl ns)          */
    long (*bands)[MAXBND+1],     /* O: Image bands                          */
    long *nbands,                /* O: Number of bands per image            */
    long *totbnd,                /* O: Total number of bands specified      */
    const long *imglimit,        /* I: Maximum images caller can handle     */
    long *same_window,           /* O: 1 if all windows the same            */
    long (*qflag)[2]             /* O: 1 user defined window & bands else
                                       0 if default window and bands        */
);

int c_getin_nocheck
(
    struct PARBLK *block,        /* I: TAE parameter block                  */
    const char *param_buf,       /* I: Input image parameter name           */
    const long *element,         /* I: Element array number of param.       */
    char tnames[][CMLEN],        /* O: TAE names of images specified        */
    char hnames[][CMLEN],        /* O: Host names of images specified       */
    long  *images,               /* O: Number of images                     */
    long  (*window)[4],          /* O: Image windows (sl ss nl ns)          */
    long  (*bands)[MAXBND+1],    /* O: Image bands                          */
    long  *nbands,               /* O: Number of bands per image            */
    long  *totbnd,               /* O: Total number of bands specified      */
    const long *imglimit,        /* I: Maximum images caller can handle     */
    long  *same_window,          /* O: 1 if all windows the same            */
    long  (*qflag)[2]            /* O: 1 user defined window & bands else
                                       0 if default window and bands        */
);

int c_getin2
(
    const char inparm[],         /* I: Input image string with              */
    char tnames[][CMLEN],        /* O: TAE names of images specified        */
    char hnames[][CMLEN],        /* O: Host names of images specified       */
    long *images,                /* O: Number of images                     */
    long (*window)[4],           /* O: Image windows (sl ss nl ns)          */
    long (*bands)[MAXBND+1],     /* O: Image bands                          */
    long *nbands,                /* O: Number of bands per image            */
    long *totbnd,                /* O: Total number of bands specified      */
    const long *imglimit,        /* I: Maximum images caller can handle     */
    long *same_window,           /* O: 1 if all windows the same            */
    long (*qflag)[2]             /* O: 1 user defined window & bands else
                                       0 if default window and bands        */
);

int c_getnam
(
    struct PARBLK *vblock,       /* I: TAE parameter block                  */
    const char *param,           /* I: TAE parameter name                   */
    const long *element,         /* I: Element of parameter                 */
    char *fname,                 /* O: Host file name to be returned        */
    const char *ext,             /* I: Default extension                    */
    const long *access           /* I: Type of I/O access                   */
);

int c_getout
(
    struct PARBLK *vblock,       /* I: TAE parameter block                  */
    const char *param,           /* I: TAE parameter name                   */
    char taeout[][CMLEN],        /* O: TAE image name(s)                    */
    char hostout[][CMLEN]        /* O: Host image file name(s)              */
);

int c_gettyp
(
    const char hosin[][CMLEN],   /* I: Input host file names                */
    const long *nmrimg,          /* I: Number of input images               */
    long  *odtype                /* O: Output data type of input img        */
);

int c_hsclos
(
    const long *fd,              /* I: File descriptor of in images hist    */
    const char hosin[]           /* I: Input host name                      */
);

long c_hsopen
(
    int   *fd,                   /* I: File descriptor of in images hist    */
    const char hosin[],          /* I: Input image name                     */
    const long *access           /* I: Type of access to input image hist   */
);

int c_hsread
(
    const long *fd,              /* I: File descriptor of in image hist     */
    char key[],                  /* I/O: Key buffer                         */
    char **buf,                  /* O: History buffer                       */
    long *len                    /* I: Char length of label service rec     */
);

int c_hstout
(
    struct PARBLK *vblock,       /* I: Pointer to TAE parameter block       */
    const char *hosout,          /* I: Image history to be updated          */
    const char (*hosin)[CMLEN],  /* I: Image names whose hist is copied     */
    const long *numin,           /* I: # of images specified in "hosin"     */
    const char *histext          /* I: Text to be included in hist record   */
);

int c_inmnmx
(
    const long *dtype,           /* I: Data type                            */
    const long *totbnd,          /* I: Total number of bands                */
    double *minval,              /* O: Minimum intensity values array       */
    double *maxval               /* O: Maximum intensity values array       */
);

int c_mkimg
(
    const char *in_string,       /* I: Character string containing filen    */
    const char *ext,             /* I: Defines string to be appended        */
    char *tname,                 /* O: String containing TAE name of im     */
    char *hname,                 /* O: String containing hostname of im     */
    const long *flag             /* I: Flag indicating existence of file    */
);

int c_mknam
(
    const char *filename,        /* I: Input file name                      */
    char *outfile,               /* O: Host file name to be returned        */
    const char *ext,             /* I: Default extension                    */
    const long *access           /* I: Type of I/O access                   */
);

#endif /* obsolete interfaces */

#endif
