#ifndef UTIL2_H
#define UTIL2_H

#include <stdio.h>
#include <signal.h>
#include "imageio.h"            /* MAXBND definition                        */
#include "cm.h"                 /* PRS_LST definition                       */
#include "ddr.h"                /* DDR definition                           */

int c_4digitdt
(
    int date                    /* I: Date in format YYJJJ to convert       */
);

int c_4digityr
(
    int year                    /* I: Year value to be converted            */
);

void las_asc2eb
(
    char *buf,                  /* I/O: String to convert,ASCII-EBCDIC      */
    int size                    /* I: Number of bytes in string             */
);

void las_complt
(
    const char *pname,          /* I: Program name                          */
    const char  tnames[][CMLEN],/* I: Output file names                     */
    int count                   /* I: Number of output files                */
);

void las_inittm
(
    const char *subject,        /* I: Subject message (process name)        */
    const char *verb,           /* I: Verb of message ('Processing')        */
    const char *entity,         /* I: Entity of display message             */
    const char *cat,            /* I: Category of message (e.g. 'band')     */
    const char *text,           /* I: Appended text string of processing    */
    int max_entity,             /* I: Maximum number of 'entity' processed  */
    int max_cat,                /* I: Maximum number of bands processed     */
    int seconds,                /* I: Number of seconds between message     */
    int flag                    /* I: Specifies format of display message
                                      ALLCAT: Processing all bands 
                                      ONECAT: Processing one band 
                                      GEN1: General display message one
                                      GEN2: General display message two     */
);

void initialize_las
(
    struct PARBLK *vblock       /* O: TAE parameter block                   */
);

void las_eb2asc
(
    char *buf,                  /* I/O: Str to convert,EBCDIC to ASCII      */
    int size                    /* I: Number of bytes in string             */
);

void c_getgbl
(
    struct PARBLK *vblock,      /* I: TAE parameter block                   */
    char histry[],              /* O: History flag                          */ 
    char minmax[],              /* O: Minmax flag                           */
    int  *msgtim                /* O: Message timing                        */
);

int las_getnum
(
   struct PARBLK *vblock,       /* I: TAE parameter block                   */
   const char parname[],        /* I: The parameter name                    */
   int   count,                 /* I: Number of entries to read in          */
   int   dtype,                 /* I: NEWLAS data type                      */
   unsigned  char  buf[]        /* O: Buffer to read into                   */
);

int c_getparm
(
    const char *filename,       /* I: Name of the project parameter file    */
    const char *parm_name,      /* I: Name of the variable retrieved        */
    char string_value[][CMLEN], /* O: String values of variables return     */
    char *type,                 /* O: Type of the parameter returned        */
    double double_value[],      /* O: Double values of the var. returned    */
    int  long_value[],          /* O: variable being returned               */
    int  position               /* I: The nth occurrence to retrieve        */
);

int las_getstr
(
    struct PARBLK *vblock,      /* I: TAE parameter block                   */
    const char parname[],       /* I: The parameter name                    */
    int count,                  /* I: Number of entries to read in          */
    int slen,                   /* I: Dimensioned length of string(s)       */
    char buf[]                  /* O: Buffer to read into                   */
);

int c_getsys
(
    char *system                /* O: System of CPU                         */
);

void las_grejul
(
    int year,                   /* I: Year                                  */
    int month,                  /* I: Gregorian month                       */
    int day,                    /* I: Gregorian day                         */
    int *jdate                  /* O: Julian date                           */
);



int c_hsetpr
(
    struct PARBLK *vblock,      /* I: TAE parameter block                   */
    int *term,                  /* O: Boolean for terminal output           */
    int *file,                  /* O: Boolean for file output               */
    int  *lp,                   /* O: Boolean for line printer output       */
    char  *prtfile,             /* I/O: Name of file or lp working file     */
    char  *filename             /* O: Name of file or lp output file        */
);

void c_index
(
    const char string1[],       /* I: String for comparison                 */
    const char string2[],       /* I: Possible substring                    */
    int  *pos                   /* O: Pos. of string2 in string1 
                                      (if not a substring)                  */
);


int las_isleap
(
    int year                    /* I: Year to test                          */
);

void las_julgre
(
    int year,                   /* I: Year                                  */
    int jdate,                  /* I: Julian date                           */
    int *month,                 /* O: Gregorian month                       */
    int *day                    /* O: Gregorian day                         */
);

void las_low2up
(
    char *buf                  /* I/O: Buffer to convert                    */
);

void las_pterm
(
    char *buf,                  /* I/O: Character buffer to output          */
    int flag                    /* I: Flag to query user for command        */
);

int c_putparm
(
    const char *filename,            /* I: Hostname of the proj param file  */
    const char *parmname,            /* I: Parameter name                   */
    const char string_value[][CMLEN],/* I: String values of the parameter   */
    const char *type,                /* I: Type of parameter to write(I,R,S)*/
    const double double_value[],     /* I: The double values of the param   */
    int  dimension,                  /* I: Dimension of the parameter       */
    const int  long_value[]          /* I: The long values of the parameter */
);

void c_sigblk
(
    int *mask                   /* I/O: Current mask that gets updated      */
);


void c_sigunblk
(
    int *mask                   /* I/O: Mask that gets reset                */
);

void c_startm(void);

void c_stoptm(void);

int las_sysset
(
    char *csys,                 /* I/O: Character coded system              */
    int *nsys,                  /* I/O: System numerical code               */
    int sysflg                  /* I: System conversion flag:               */
                                /*    SETN: 0 convert sys str to value      */
                                /*    SETS: 1 convert sys value to str      */
);

void c_txtout
(
    char  buf[],                /* I: Text to be written out                */
    const int   print[],        /* I: Print options (FILE, LP, TERM)        */
    FILE  *fp                   /* I: File descrip for print FILE or LP     */
                                /*    if fp == 0 then no file open          */
);

void las_up2low
(
    char *buf                  /* I/O: Buf converted from upper/lower       */
);

int getusern
(
    char *username              /* I/O: User name                           */ 
);

int logit
(
    const char *progname        /* I/O: Program name to be logged           */
);



#ifndef NO_OBSOLETE_INTERFACE


void c_asc2eb
(
    char *buf,                  /* I/O: string to convert,ASCII-EBCDIC      */
    const long *size            /* I: Number of bytes in string             */
);

void c_complt
(
    const char *pname,          /* I: Program name                          */
    char  tnames[][CMLEN],      /* I: Output file names                     */
    const long  *count          /* I: Number of output files                */
);

void c_inittm
(
    const char *subject,        /* I: Subject message (process name)        */
    const char *verb,           /* I: Verb of message ('Processing')        */
    const char *entity,         /* I: Entity of display message             */
    const char *cat,            /* I: Category of message (e.g. 'band')     */
    const char *text,           /* I: Appended text string of processing    */
    const long *max_entity,     /* I: Maximum number of 'entity' proces     */
    const long *max_cat,        /* I: Maximum number of bands processed     */
    const long *seconds,        /* I: Number of seconds between message     */
    const long *flag            /* I: Specifies format of display message
                                      ALLCAT: Processing all bands 
                                      ONECAT: Processing one band 
                                      GEN1: General display message one
                                      GEN2: General display message two     */
);

void c_inlas
(
    struct PARBLK *vblock,      /* O: TAE parameter block                   */
    long *sout                  /* O: Unit number for opening stdout        */
);

void c_eb2asc
(
    char *buf,                  /* I/O: str to convert, EBCDIC to ASCII     */
    const long *size            /* I: Number of bytes in string             */
);

int c_getnum
(
    struct PARBLK *vblock,      /* I: TAE parameter block                   */
    const char parname[],       /* I: The parameter name                    */
    const long *count,          /* I: Number of entries to read in          */
    const long *dtype,          /* I: NEWLAS data type                      */
    unsigned char  buf[]        /* O: Buffer to read into                   */
);

int c_getstr
(
    struct PARBLK *vblock,      /* I: TAE parameter block                   */
    const char parname[],       /* I: The parameter name                    */
    const long *count,          /* I: Number of entries to read in          */
    const long *slen,           /* I: Dimensioned length of string(s)       */
    char buf[]                  /* O: Buffer to read into                   */
);

void c_grejul
(
    const long *year,           /* I: Year                                  */
    const long *month,          /* I: Gregorian month                       */
    const long *day,            /* I: Gregorian day                         */
    long *jdate                 /* O: Julian date                           */
);


int c_isleap
(
    const long *year            /* I: Year to test                          */
);

void c_julgre
(
    const long *year,           /* I: Year                                  */
    const long *jdate,          /* I: Julian date                           */
    long *month,                /* O: Gregorian month                       */
    long *day                   /* O: Gregorian day                         */
);

void c_low2up
(
    char *buf,                  /* I/O: Buffer to convert                   */
    const long *size            /* I: Size of string in buffer              */
);

void c_pterm
(
    char *buf,                  /* I: Character buffer to output            */
    const long *flag            /* I: Flag to query user for command        */
);

int c_sysset
(
    char *csys,                 /* I/O: Character coded system              */
    long *nsys,                 /* I/O: System numerical code               */
    const long *sysflg          /* I: System conversion flag:               */
                                /*    SETN: 0 convert sys str to value      */
                                /*    SETS: 1 convert sys value to str      */
);

void c_up2low
(
    char *buf,                  /* I/O: Buf converted from upper/lower      */
    const long *size            /* I: Number of bytes to convert in buf     */
);

#endif /* obsolete interfaces   */

#endif



